// INPUT_4_DIP
//
// Prova ingressi con DipSwitch ed elaborazione
// dati utilizzando mascherature binarie (esadecimali)
//
// ITS AUTOMA20
//
// Lezione 3: 21/06/2021
//

// Definizione uscite
#define LED_VERDE   12
#define LED_ROSSO   13
#define LED_BLU     11
// Definizione ingressi DipSwitch
#define INT_1    4
#define INT_2    5
#define INT_3    6
#define INT_4    7
// Definizione stato attivo ingressi
// E' lo zero poichè il dip-switch chiude
// il contatto verso massa (GND o ground)
#define INT_ATTIVO     0
#define INT_NON_ATTIVO (! INT_ATTIVO)
// Definizione stato attivo uscite
// E' l'uno poichè il pin di Arduino 
// fornisce tensione (e corrente) in uscita
// per accendere il LED
#define LED_ATTIVO     1
#define LED_NON_ATTIVO (! LED_ATTIVO)
// Variabili utilizzate nel programma
int Valore;
bool LedVerde, LedRosso, LedBlu;

void setup()
{
  // Configurazione pin di uscita
  pinMode( LED_VERDE, OUTPUT );	
  pinMode( LED_ROSSO, OUTPUT );
  pinMode( LED_BLU, OUTPUT );
  // Configurazione pin di ingresso
  pinMode( INT_1, INPUT );
  pinMode( INT_2, INPUT );
  pinMode( INT_3, INPUT );
  pinMode( INT_4, INPUT );  
  // Inizializzazione delle variabili
  // ad un valore conosciuto e desiderato
  LedVerde = LED_NON_ATTIVO;
  LedRosso = LED_NON_ATTIVO;
  LedBlu = LED_NON_ATTIVO;
  // La variabile seguente non necessariamente
  // doveva essere azzerata, poichè ad ogni giro
  // di LOOP() viene comunque azzerata, però
  // è buona abitudine di programmazione farlo...
  Valore = 0;
  // Imposizione del valore sulle uscite
  // all'accensione della scheda
  digitalWrite( LED_VERDE, LedVerde );
  digitalWrite( LED_ROSSO, LedRosso );
  digitalWrite( LED_BLU, LedBlu );
}

void loop()
{
    // ***************************************************
	// Lettura ingressi (dal mondo esterno)
    // ***************************************************
    // Gli stati di tutti gli ingressi vengono 
    // memorizzati in una sola variabile
    // (molto più comodo per l'elaborazione)
    Valore = 0;
    // In base allo stato di ogni singolo ingresso
    // viene attivato 1 solo bit nella variabile di appoggio
    // Controllo sul primo dip-switch
    if( digitalRead( INT_1 ) == INT_ATTIVO ) 
      Valore = Valore |  0x01; 
    else
      Valore = Valore & ~0x01; 
    // Stesso controllo sul secondo dip-switch (contraendo la
    // scrittura, come permette di fare il linguaggio C)
    if( digitalRead( INT_2 ) == INT_ATTIVO ) 
      Valore |=  0x02; 
    else
      Valore &= ~0x02; 
    if( digitalRead( INT_3 ) == INT_ATTIVO ) 
      Valore |=  0x04; 
    else
      Valore &= ~0x04; 
    if( digitalRead( INT_4 ) == INT_ATTIVO ) 
      Valore |=  0x08; 
    else
      Valore &= ~0x08;   
    //
    // ***************************************************
    // Elaborazione dei dati
    // ***************************************************
    // Il led rosso e verde corrispondono ai primi due bit
    // a sinistra della variabile di appoggio
    if( Valore & 0x01 ) LedVerde = true; else LedVerde = false;
    // Si poteva anche ulteriormente semplificare, questa 
    // scrittura equivale alla riga precedente, con riferimenti
    // al Led Rosso
    LedRosso = Valore & 0x02;
    // Il terzo led corrisponde a un controllo sulla
    // combinazione dei primi due, associata allo stato
    // del terzo e quarto dip-switch
    // NOTA: La mascheratura viene effettuata su entrambi
    // i bit insieme !!! Quindi la maschera è 0000-1100 ==> 0x0C
    if( (Valore & 0x0C) == 0x04 )
    {
      // Terzo dip-switch alto allora il Led Blu indica
      // quando gli altri due led sono diversi
      if( LedRosso != LedVerde ) 
       { LedBlu = true; }
      else 
       { LedBlu = false; }
    }
    else if( (Valore & 0x0C) == 0x08 )
    {
      // Quarto dip-switch alto allora il Led Blu indica
      // quando gli altri due led sono uguali
      if( LedRosso == LedVerde )
      { LedBlu = true; }
      else 
      { LedBlu = false; }     
    }
    else if( (Valore & 0x0C) == 0x0C )
    {  
      // Terzo e quarto dip-switch alti allora il Led Blu 
      // sarà sempre acceso
      LedBlu = true;
    }
    else
    {
      // Ultimo controllo ovviamente è la quarta combinazione
      // mancante, per cui non occorre effettuare mascherature
      // od ulteriori controlli...
      // Terzo e quarto dip-switch bassi allora il Led Blu 
      // sarà sempre spento
      LedBlu = false;      
    }
    // ***********************************************
    // Scrittura uscite (verso il mondo esterno)
    // ***********************************************
    digitalWrite( LED_VERDE, LedVerde );
    digitalWrite( LED_ROSSO, LedRosso );  
    digitalWrite( LED_BLU, LedBlu ); 
}























